"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DurationPickerState = exports.DurationPickerFieldsState = void 0;
const core_1 = require("@tangential/core");
const moment = require("moment/moment");
class DurationPickerFieldsState {
    constructor(picker, unit) {
        this.picker = picker;
        this.index = 0;
        this.unit = core_1.TimeUnits.h;
        this.selected = false;
        this.unit = unit;
        this.selected = false;
    }
    get label() {
        return this.unit.label;
    }
    canSelect() {
        let can;
        const count = this.picker.selectedCount();
        if (count >= 3) {
            can = false;
        }
        else if (count === 0) {
            can = true;
        }
        else {
            can = this.previous?.selected || this.next?.selected || false;
        }
        return can;
    }
    canDeselect() {
        /* If next and previous are selected then you cannot deselect this one. */
        return (!this.previous || !this.next || !this.previous.selected || !this.next.selected);
    }
    canToggleSelection() {
        return this.selected ? this.canDeselect() : this.canSelect();
    }
    get value() {
        let result;
        if (this.previous && this.previous.selected) {
            result = this.picker.duration.get(this.unit.momentKey);
        }
        else {
            result = this.picker.duration.as(this.unit.momentKey);
        }
        return Math.floor(result);
    }
    set value(val) {
        if (val >= 0) {
            const prev = this.value;
            this.picker.duration.add(val - prev, this.unit.momentKey);
        }
    }
    get separator() {
        return this.unit.separatorSuffix;
    }
    get max() {
        return (this.previous && this.previous.selected) ? this.unit.logicalMax : 9999;
    }
}
exports.DurationPickerFieldsState = DurationPickerFieldsState;
/** @todo: ggranum: Remove use of MomentJS if possible. */
class DurationPickerState {
    constructor() {
        this.fields = [];
        this.fieldsByKey = {};
        this.duration = moment.duration(0);
        this.max = 100;
        this.fields = [
            new DurationPickerFieldsState(this, core_1.TimeUnits.day), new DurationPickerFieldsState(this, core_1.TimeUnits.h),
            new DurationPickerFieldsState(this, core_1.TimeUnits.min), new DurationPickerFieldsState(this, core_1.TimeUnits.s),
            new DurationPickerFieldsState(this, core_1.TimeUnits.ms),
        ];
        for (let i = 0; i < this.fields.length; i++) {
            const f = this.fields[i];
            f.next = this.fields[i + 1];
            f.previous = this.fields[i - 1];
            f.index = i;
            this.fieldsByKey[f.unit.unitKey] = f;
        }
    }
    selectedFields() {
        return this.fields.filter(f => f.selected);
    }
    selectedCount() {
        return this.selectedFields().length;
    }
    setDuration(duration) {
        this.duration = duration;
    }
    select(unit) {
        this.fieldsByKey[unit.unitKey].selected = true;
    }
    get millis() {
        return this.duration.asMilliseconds();
    }
}
exports.DurationPickerState = DurationPickerState;
//# sourceMappingURL=duration-picker-state.js.map